@@ -267,11 +267,15 @@ class LLMInterface:
         }
         x = ""
         for x in ans2:
-            if self.lt is not None and hasattr(self.llm, "cur_tool_spec"):
+            if self.lt is not None and (
+                hasattr(self.llm, "cur_tool_spec")
+                or hasattr(self.llm, "cur_tool_specs")
+            ):
                 extra_info = {
                     "metadata": {"title": "🛠️", "status": "pending"},
                     "content": ", ".join([x["tool_name"] for x in self.lt.invoke_log]),
                 }
+            self.history_log[chat_id] = history + [msg, x]
             yield self._format_msg(x, msg, ui_history, extra_info=extra_info)
         # initial_ans = self._format_msg(x, msg, ui_history)
         # yield initial_ans
@@ -311,6 +315,7 @@ class LLMInterface:
 
             for x in ans2:
                 # pass
+                self.history_log[chat_id] = history + [msg, x]
                 yield self._format_msg(x, msg, ui_history)
             # yield self._format_msg(x, msg, ui_history)
 
