@@ -19,9 +19,11 @@ from .llm_providers.maritaca import LLM_Maritalk
 from .llm_providers.aws_bedrock_nova import LLM_Nova_Bedrock
 from .llm_providers.deepseek import LLM_Deepseek
 from .llm_providers.grok import LLM_Grok
+from .llm_providers.aws_bedrock_via_openai import LLM_Bedrock_OpenAI
 
 # local
 from .llm_providers.ollama import LLM_Ollama
+from .llm_providers.vllm import LLM_VLLM
 
 
 warnings.simplefilter("always", DeprecationWarning)
@@ -34,8 +36,14 @@ class LLM_Provider:
         "Claude 3 Opus - Anthropic",
         "Claude 3 Haiku - Anthropic",
         "Claude 3 Haiku - Bedrock",
+        "Claude 3.5 Haiku - Anthropic",
+        "Claude 3.5 Haiku - Bedrock",
         "Claude 3 Sonnet - Bedrock",
         "Claude 3 Opus - Bedrock",
+        "Claude 4 Sonnet - Anthropic",
+        "Claude 4 Sonnet - Bedrock",
+        "Claude 4 Opus - Anthropic",
+        "Claude 4 Opus - Bedrock",
         "Claude 2.1",
         "Claude Instant 1.2",
         "Llama2 13b",
@@ -48,42 +56,10 @@ class LLM_Provider:
         "GPT 4.1 - OpenAI",
         "GPT 3.5 - OpenAI",
         "GPT 4o mini - OpenAI",
-    ]
-
-    allowed_llms = [
-        # Local
-        "OpenAI GPT OSS 20b - Ollama",
-        "OpenAI GPT OSS 120b - Ollama",
-        "Qwen 3 0.6b - Ollama",
-        "Qwen 3 1.7b - Ollama",
-        "Qwen 3 4b - Ollama",
-        "Qwen 3 8b - Ollama",
-        "Qwen 3 14b - Ollama",
-        "Qwen 3 Coder 30b - Ollama",
-        "Llama4 16x17b - Ollama",
-        "Qwen 2.5vl 7b - Ollama",
-        "DeepSeek R1 14b - Ollama",
         # AWS
         "Amazon Nova Micro 1.0 - Bedrock",
         "Amazon Nova Lite 1.0 - Bedrock",
         "Amazon Nova Pro 1.0 - Bedrock",
-        # Grok
-        "Grok4 - Grok",
-        # Maritaca
-        "Sabia3 - Maritaca",
-        # OpenAI
-        "GPT 5 - OpenAI",
-        "GPT 5 mini - OpenAI",
-        "GPT 5 nano - OpenAI",
-        # Anthropic
-        "Claude 4 Opus - Anthropic",
-        "Claude 4 Sonnet - Anthropic",
-        "Claude 4 Opus - Bedrock",
-        "Claude 4 Sonnet - Bedrock",
-        "Claude 3.7 Sonnet - Anthropic",
-        "Claude 3.5 Haiku - Anthropic",
-        "Claude 3.7 Sonnet - Bedrock",
-        "Claude 3.5 Haiku - Bedrock",
         # Misc
         "DeepSeekV3 Chat - DeepSeek",
         "Command R - Bedrock",
@@ -93,6 +69,47 @@ class LLM_Provider:
         "Llama3_1 8b instruct",
         "Llama3_1 70b instruct",
         "Llama3_1 405b instruct",
+        # Qwen
+        "Qwen 2.5vl 7b - Ollama",
+        "Qwen 2.5vl 3b - Ollama",
+    ]
+
+    allowed_llms = [
+        # Local
+        "Qwen 3 0.6b - Ollama",
+        "OpenAI GPT OSS 20b - Ollama",
+        "OpenAI GPT OSS 120b - Ollama",
+        "Qwen 3 1.7b - Ollama",
+        "Qwen 3 4b - Ollama",
+        "Qwen 3 8b - Ollama",
+        "Qwen 3 14b - Ollama",
+        "Qwen 3 Coder 30b - Ollama",
+        "Llama4 16x17b - Ollama",
+        "Qwen 3vl 8b - Ollama",
+        "Qwen 3vl 4b - Ollama",
+        "Qwen 3vl 2b - Ollama",
+        "DeepSeek R1 14b - Ollama",
+        "Qwen 3 1.7b - VLLM",
+        # AWS Bedrock via OpenAI API
+        "OpenAI GPT OSS 20b - AWSBedrock_OpenAI",
+        "OpenAI GPT OSS 120b - AWSBedrock_OpenAI",
+        # Grok
+        "Grok4 - Grok",
+        "Grok4 Fast reasoning - Grok",
+        "Grok4 Fast nonreasoning - Grok",
+        # Maritaca
+        "Sabia3 - Maritaca",
+        # OpenAI
+        "GPT 5 - OpenAI",
+        "GPT 5 mini - OpenAI",
+        "GPT 5 nano - OpenAI",
+        # Anthropic
+        "Claude 4.5 Sonnet - Anthropic",
+        "Claude 4.5 Sonnet - Bedrock",
+        "Claude 4.5 Haiku - Anthropic",
+        "Claude 4.5 Haiku - Bedrock",
+        "Claude 3.7 Sonnet - Anthropic",
+        "Claude 3.7 Sonnet - Bedrock",
     ]
 
     def get_llm(bedrock_client, llm):
@@ -137,11 +154,29 @@ class LLM_Provider:
             return LLM_Ollama(model="Qwen 3 8b Ollama")
         elif llm == "Qwen 3 14b - Ollama":
             return LLM_Ollama(model="Qwen 3 14b Ollama")
+        elif llm == "Qwen 3vl 8b - Ollama":
+            return LLM_Ollama(model="Qwen 3vl 8b Ollama")
+        elif llm == "Qwen 3vl 4b - Ollama":
+            return LLM_Ollama(model="Qwen 3vl 4b Ollama")
+        elif llm == "Qwen 3vl 2b - Ollama":
+            return LLM_Ollama(model="Qwen 3vl 2b Ollama")
         elif llm == "Qwen 2.5vl 7b - Ollama":
             return LLM_Ollama(model="Qwen 2.5vl 7b Ollama")
+        elif llm == "Qwen 2.5vl 3b - Ollama":
+            return LLM_Ollama(model="Qwen 2.5vl 3b Ollama")
         elif llm == "DeepSeek R1 14b - Ollama":
             return LLM_Ollama(model="DeepSeek R1 14b Ollama")
 
+        # Local - vllm
+        elif llm == "Qwen 3 1.7b - VLLM":
+            return LLM_VLLM(model="Qwen 3 1.7b VLLM")
+
+        # AWS Bedrock via OpenAI API
+        elif llm == "OpenAI GPT OSS 20b - AWSBedrock_OpenAI":
+            return LLM_Bedrock_OpenAI(model_size="GPT OSS 20b Bedrock")
+        elif llm == "OpenAI GPT OSS 120b - AWSBedrock_OpenAI":
+            return LLM_Bedrock_OpenAI(model_size="GPT OSS 120b Bedrock")
+
         # Amazon
         elif llm == "Amazon Nova Micro 1.0 - Bedrock":
             return LLM_Nova_Bedrock(bedrock_client, model_size="Nova_Micro")
@@ -159,6 +194,10 @@ class LLM_Provider:
             return LLM_Grok(model_size="Grok2Vision xAI")
         elif llm == "Grok4 - Grok":
             return LLM_Grok(model_size="Grok4")
+        elif llm == "Grok4 Fast reasoning - Grok":
+            return LLM_Grok(model_size="Grok4-fast-reasoning")
+        elif llm == "Grok4 Fast nonreasoning - Grok":
+            return LLM_Grok(model_size="Grok4-fast-non-reasoning")
 
         # DeepSeek
         elif llm == "DeepSeekV3 Chat - DeepSeek":
@@ -204,10 +243,18 @@ class LLM_Provider:
         # current Claude
         elif llm == "Claude 4 Sonnet - Anthropic":
             return LLM_Claude_Anthropic(model_size="Sonnet 4 Anthropic")
+        elif llm == "Claude 4.5 Sonnet - Anthropic":
+            return LLM_Claude_Anthropic(model_size="Sonnet 4.5 Anthropic")
+        elif llm == "Claude 4.5 Haiku - Anthropic":
+            return LLM_Claude_Anthropic(model_size="Haiku 4.5 Anthropic")
         elif llm == "Claude 4 Opus - Anthropic":
             return LLM_Claude_Anthropic(model_size="Opus 4 Anthropic")
         elif llm == "Claude 4 Sonnet - Bedrock":
             return LLM_Claude_Bedrock(bedrock_client, model_size="Sonnet 4")
+        elif llm == "Claude 4.5 Sonnet - Bedrock":
+            return LLM_Claude_Bedrock(bedrock_client, model_size="Sonnet 4.5")
+        elif llm == "Claude 4.5 Haiku - Bedrock":
+            return LLM_Claude_Bedrock(bedrock_client, model_size="Haiku 4.5")
         elif llm == "Claude 4 Opus - Bedrock":
             return LLM_Claude_Bedrock(bedrock_client, model_size="Opus 4")
         elif llm == "Claude 3.7 Sonnet - Bedrock":
@@ -225,11 +272,11 @@ class LLM_Provider:
 
         # OpenAI
         elif llm == "GPT 5 - OpenAI":
-            return LLM_GPT_OpenAI(model_size="GPT5 OpenAI")
+            return LLM_GPT_OpenAI(model_size="GPT5 OpenAI", reasoning_effort="low")
         elif llm == "GPT 5 mini - OpenAI":
-            return LLM_GPT_OpenAI(model_size="GPT5 mini OpenAI")
+            return LLM_GPT_OpenAI(model_size="GPT5 mini OpenAI", reasoning_effort="low")
         elif llm == "GPT 5 nano - OpenAI":
-            return LLM_GPT_OpenAI(model_size="GPT5 nano OpenAI")
+            return LLM_GPT_OpenAI(model_size="GPT5 nano OpenAI", reasoning_effort="low")
         elif llm == "GPT 4.1 - OpenAI":
             return LLM_GPT_OpenAI(model_size="GPT4_1 OpenAI")
         elif llm == "GPT 3.5 - OpenAI":
