@@ -1,6 +1,7 @@
 import re
 import json
 import time
+import types
 
 import anthropic
 from .base_service import LLM_Service
@@ -22,6 +23,18 @@ class LLM_Claude_Anthropic(LLM_Service):
             )
             self.price_per_M_input_tokens = 15
             self.price_per_M_output_tokens = 75
+        elif model_size == "Sonnet 4.5 Anthropic":
+            self.model_id = "claude-sonnet-4-5-20250929"
+            self.llm_description = "Anthropic Claude 4.5 Sonnet (Medium-size LLM) - directly from Anthropic"
+            self.price_per_M_input_tokens = 3
+            self.price_per_M_output_tokens = 15
+        elif model_size == "Haiku 4.5 Anthropic":
+            self.model_id = "claude-haiku-4-5-20251001"
+            self.llm_description = (
+                "Anthropic Claude 4.5 Haiku (Small-size LLM) - directly from Anthropic"
+            )
+            self.price_per_M_input_tokens = 1
+            self.price_per_M_output_tokens = 5
         elif model_size == "Sonnet 4 Anthropic":
             self.model_id = "claude-sonnet-4-20250514"
             self.llm_description = (
@@ -64,7 +77,7 @@ class LLM_Claude_Anthropic(LLM_Service):
         self.config = {
             # "messages": prompt,
             # "system": sysprompt,
-            "max_tokens": 4000,
+            "max_tokens": 8192,
             "temperature": 0.5,  # 0.5 is default,
             "stream": True,
             # "top_k": 250,
@@ -184,6 +197,10 @@ class LLM_Claude_Anthropic(LLM_Service):
                             return_results_only=True,
                             **self.cur_tool_spec["input"],
                         )
+                        if isinstance(tool_ans, types.GeneratorType):
+                            for partial_ans in tool_ans:
+                                yield partial_ans
+                            tool_ans = partial_ans
 
                         # append assistant responses
                         assistant_msg = {"role": "assistant", "content": []}
@@ -269,6 +286,9 @@ class LLM_Claude_Anthropic(LLM_Service):
                 yield postpend + cur_ans + stop_txt
                 break
         if cur_tool_spec is not None:
-            cur_tool_spec["input"] = json.loads(cur_tool_spec["input"])
+            if cur_tool_spec["input"].strip() == "":
+                cur_tool_spec["input"] = {}
+            else:
+                cur_tool_spec["input"] = json.loads(cur_tool_spec["input"])
         self.cur_tool_spec = cur_tool_spec
         self.stop_reason = stop_reason
