@@ -1,6 +1,7 @@
 import re
 import json
 import time
+import types
 
 from typing import Dict, List
 from .base_service import LLM_Service
@@ -28,6 +29,20 @@ class LLM_Claude_Bedrock(LLM_Service):
             )
             self.price_per_M_input_tokens = 3
             self.price_per_M_output_tokens = 15
+        elif model_size == "Sonnet 4.5":
+            self.model_id = "us.anthropic.claude-sonnet-4-5-20250929-v1:0"
+            self.llm_description = (
+                "Anthropic Claude 4.5 Sonnet from AWS Bedrock (Medium-size LLM)"
+            )
+            self.price_per_M_input_tokens = 3
+            self.price_per_M_output_tokens = 15
+        elif model_size == "Haiku 4.5":
+            self.model_id = "us.anthropic.claude-haiku-4-5-20251001-v1:0"
+            self.llm_description = (
+                "Anthropic Claude 4.5 Haiku from AWS Bedrock (Small-size LLM)"
+            )
+            self.price_per_M_input_tokens = 1
+            self.price_per_M_output_tokens = 5
         elif model_size == "Opus 4":
             self.model_id = "us.anthropic.claude-opus-4-20250514-v1:0"
             self.llm_description = (
@@ -82,7 +97,7 @@ class LLM_Claude_Bedrock(LLM_Service):
         self.config = {
             # "messages": prompt,
             # "system": sysprompt,
-            "max_tokens": 4000,
+            "max_tokens": 8192,
             "temperature": 0.5,  # 0.5 is default,
             # "top_k": 250,
             # "top_p": 1,
@@ -196,6 +211,10 @@ class LLM_Claude_Bedrock(LLM_Service):
                             return_results_only=True,
                             **self.cur_tool_spec["input"],
                         )
+                        if isinstance(tool_ans, types.GeneratorType):
+                            for partial_ans in tool_ans:
+                                yield partial_ans
+                            tool_ans = partial_ans
 
                         # append assistant responses
                         assistant_msg = {"role": "assistant", "content": []}
